/****************************** clr.c **************************************
*                                                                          *
*  by David Fredericks, Pathfinder Associates                              *
*                                                                          *
*  Compiled and tested with:  Borland Turbo C version 2.0                  *
*                             Microsoft C version 5.1                      *
*                             Watcom C version 7.0                         *
*                                                                          *
*  version 1, 5/31/89                                                      *
*                                                                          *
*  There are NO restrictions on the use of this code as long               *
*                             as authorship is acknowledged.               *
*  usage:  CLR                                                             *
*                                                                          *
*  This program clears the complete screen using the number of lines       *
*  from the BIOS data area.                                                *
*                                                                          *
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

union   REGS    gregs;                  /* registers for int86 calls        */

#define BIOS_VIDEO  0x10

                  /*- BIOS Data Area locations -*/
char far *BiosRows = (char far *)0x00400084L;   /* number of screen rows - 1*/
char far *BiosCols = (char far *)0x0040004AL;   /* number of screen columns */

                    /*- BIOS Video Functions -*/
#define SETCURSOR   0x02                /* set cursor position              */
#define SCRUP       0x06                /* scroll the screen up             */
#define RDCHARATTR  0x08                /* read character and attribute     */
#define GET_MODE    0x0F                /* get current video mode           */
#define ALT_SELECT  0x12                /* alternate select                 */
#define DCC         0x1A                /* display combination code         */

                     /*- BIOS Video Subfunctions -*/
#define VID_CFG     0x10                /* get video configuration (EGA)    */
#define GET_DCC     0x00                /* get display combination code     */

#define EGA         0x01                /* Enhanced Graphics Adapter        */
#define VGA         0x02                /* Video Graphics Array             */
int DisplayType (void);

void main()
{
    int rows;

    /* step 1, get row count */
    if (*BiosRows < 24)          /* Monochrome often has count of 0    */
        rows = 24;               /* 25 lines - 1 (count starts from 0) */
    else
        rows = *BiosRows;

    /* step 2, fetch the video attribute for step 3 */
    gregs.h.ah = RDCHARATTR;
    gregs.h.bh = 0;
    int86(BIOS_VIDEO, &gregs, &gregs);  /* get attribute for scroll */

    /* step 3, clear the entire screen */
    gregs.h.bh = gregs.h.ah;    /* attribute from cursor location */
    gregs.h.ah = SCRUP;
    gregs.h.al = 0; /* indicate that we want screen cleared, not scrolled */
    gregs.h.ch = 0;                     /* top row      */
    gregs.h.cl = 0;                     /* left column  */
    gregs.h.dh = rows;                  /* bottom row   */
    gregs.h.dl = *BiosCols - 1;         /* right column */
    int86(BIOS_VIDEO, &gregs, &gregs);  /* scroll up */

    /* step 4, move the cursor to the beginning of the screen */
    gregs.h.ah = SETCURSOR;
    gregs.h.bh = 0;
    gregs.h.dh = 0;
    gregs.h.dl = 0;
    int86(BIOS_VIDEO, &gregs, &gregs);  /* move cursor to top left */
}
