// *************************  DYNARRAY.CXX *************************
// * The advantages of an array, with the dynamic flexibilty of a
// * linked list. Adapted from "Using C++" by Bruce Eckel
// * (c) Copryright 1989, Osborne/McGraw-Hill
// *****************************************************************

#include "dynarray.hxx"
#include <stream.hxx>
#include <stdlib.h>  // exit()

void DynArray::error(char * msg)
{
  cerr << "DynArray error: " << msg << "\n";
  exit(1);
}

DynArray::DynArray()
{
  array = new void*[chunk];  // allocate a chunk of void *'s
  for(int i = 0; i < chunk; i++)
    array[i] = (void *)0;  // zero them
  size = chunk;
  cursor = 0;
}

// The destructor frees the dynamically-allocated memory:
DynArray::~DynArray() 
{
  delete array;
}

int DynArray::add(void * new_element)
{
  // Put it in the first empty space avaialable:
  for(int i = 0; i < size; i++)
    if(array[i] == (void *)0)
    {
      array[i] = new_element;
      return i;
    }

  // at this point, no space was found.  Add new space:

  int tempsize = size + chunk;  // increase space by chunk
  void ** temp = new void*[tempsize];

  // copy the old array over:
  for(i = 0; i < size; i++)
    temp[i] = array[i];

  // initialize the rest of the new array elements:
  for( ; i < tempsize; i++)
    temp[i] = (void *)0;
  temp[i = size] = new_element;  // put at start of empty space

  delete array;  // free old memory
  array = temp;
  size = tempsize;
  return i;
}

int DynArray::remove(void * rp)
{
  for(int i = 0; i < size; i++)
    if( array[i] == rp )
    {
      array[i] = (void *)0;
      return 1;
    }
  return 0; // not found
}

int DynArray::remove(int ri)
{
  if( ri < 0 || ri >= size)
    error("remove index out of range");

  // check to see if there's an element at that slot:
  if( array[ri] )
  {
    array[ri] = (void *)0;
    return 1;
  }
  return 0; // not found
}

void DynArray::reset()
{
  cursor = 0;
  while(array[cursor] == (void *)0)
    cursor++;  // find the first nonzero element
}

void * DynArray::next()
{
  if( cursor == size -1 ) // last element, no next.
    return (void *)0;
  // Not at the end. Increment until you find a non-empty slot
  // or the end:
  while(array[++cursor] == (void *)0)
    if ( cursor == size -1 )  // no more elements in list
      return (void *)0;
  return array[cursor];
}

void * DynArray::operator[](int x)
{
  if(x < 0 || x >= size)
    error("operator[] -- index out of range");
  return array[x];  // even if it's empty...
}

int DynArray::count()
{
  int cnt = 0;
  for(int x=0; x < size; x++)
    if(array[x])
      cnt++;
  return cnt;
}