// ***************************** DYNARRAY.HXX ****************************
// *  Header for Dynarray.cxx  Dynamically-sized array of object pointers
// *  Adapted from "Using C++" by Bruce Eckel, Osborne/McGraw-Hill 1989
// ***********************************************************************

#ifndef DYNARRAY_HXX
#define DYNARRAY_HXX

// how much to increase the array when it runs out of space:
#define chunk 10  // "const" won't work with Zortech 1.07 and before

class DynArray
{
  void ** array;  // starting address of array of void pointers
  int size;       // current size of the array
  int cursor;     // to index through the array
  void error(char * msg = "");

public:
  DynArray();
  ~DynArray();
  int add(void *);  // stash an element in the array,
  // increasing the size if necessary.  Returns the index.

  // Both remove() functions return 1 if successful:
  int remove(void *);  // remove an element by pointer
  int remove(int);     // remove an element using its index
  // reset the cursor to the first nonzero element at the
  // top of the array:
  void reset();
  void * next(); // return the pointer to the next element
                 // (empty elements are skipped).  Returns NULL
                 // at end of array.
  int index() { return cursor; } // index of current element

  // return the pointer to the current element:
  void * current() { return array[cursor]; }
  void * operator[](int); // element selection
  int count();            // number of "live" items in the array
};

#endif DYNARRAY_HXX