/****** Cscape Windowing Library Demo Code *********************
 * Author: John Rex
 * Usage: edit <in >out
 * Function: Using the C-scape routines, a full-screen
 *           editor is implemented.
 * Compiler: Turbo C 2.0
 **************************************************************/	

#include <stdio.h>
#include <stdlib.h>

#include <cscape.h>		/* standard C-scape header files */
#include <teddecl.h>
#include <pcscan.h>

main()
{
	menu_type	menu;
	sed_type	sed;
	int 		ret;
	boolean		move_sed(sed_type sed, int scancode);

	disp_Init(def_ModeText, NULL);		/* start C-scape */

	menu = menu_Open();					/* create a menu object */

	/* menu_Printf() adds a field to our menu object.  The choice of
	   ted_funcs for the field's manager automatically creates a
	   text editor field.  menu_Flush() is a cleanup routine. 
	 */
	menu_Printf(menu, "@f[]", NULL, &ted_funcs);
	menu_Flush(menu);

	sed = sed_Open(menu);				/* put menu into a sed */
	ted_SetMaxSize(sed, 0L);			/* 0L means unlimited data */

	ted_ReadFile(sed, stdin);			/* read stdin into sed */
	sed_SetBorder(sed, bd_box);			/* give it a simple border */

	sed_SetPosition(sed, 0, 0);			/* set sed's location */
	sed_SetHeight(sed, 23);				/* set sed's size */	
	sed_SetWidth(sed, 78);
	sed_SetSpecial(sed, move_sed);		/* add move_sed() to list
										   of routines that manage
										   keystrokes
										 */
	
	sed_Repaint(sed);					/* display the sed */
	ret = sed_Go(sed);					/* and use it */

	ted_WriteFile(sed, stdout, TED_SOFT);	/* save edited text */

	menu_Close(menu);					/* shutdown */
	sed_Close(sed);
	disp_Close();
}

/* a sample "add-on" keystroke manager.  The call to sed_SetSpecial()
   (done above) tells C-scape to let this routine examine any
   keystrokes passed to the sed.  We will trap F1:F4 and Ctrl-F1:F4
   and use them to change the sed's size and location.
 */

boolean move_sed(sed_type sed, int scancode)
{
	int row, col, rowdel, coldel;
	int height, width, heightdel, widthdel;

	rowdel = coldel = 0;
	heightdel = widthdel = 0;

	switch(scancode) {
		/* F1 thru F4 will change location */
		case FN1:
			rowdel = -1;
			break;

		case FN3:
			rowdel = 1;
			break;

		case FN2:
			coldel = 1;
			break;

		case FN4:
			coldel = -1;
			break;

		/* Ctrl-F1 thru Ctrl-F1 will change size */
		case CTRL_FN1:
			heightdel = -1;
			break;

		case CTRL_FN3:
			heightdel = 1;
			break;

		case CTRL_FN2:
			widthdel = 1;
			break;

		case CTRL_FN4:
			widthdel = -1;
			break;
		
		/* none of the above--return FALSE to indicate
		   that we didn't use the keystroke
		 */
		default:
			return(FALSE);
	}

	/* the routines used in the following code do exactly
	   what their names suggest they do.
	 */
	if (rowdel || coldel) {
		sed_GetPosition(sed, &row, &col);
		row += rowdel;
		col += coldel;
		sed_SetPosition(sed, row, col);
	}

	else if (heightdel) {
		height = sed_GetHeight(sed) + heightdel;
		if (height >= 1 )
			sed_SetHeight(sed, height);
	}

	else if (widthdel) {
		width = sed_GetWidth(sed) + widthdel;
		if (width >= 1)
			sed_SetWidth(sed, width);
	}

	return(TRUE);	/* tell caller that we "ate" the keystroke */
}

