/**********************************************************************
 * envcmd.c : Take command line arguments from an environment variable
 * Version  : 1.0
 * Author   : Susan Greiman
 * Compiler : Turbo C 2.0
 *            Microsoft C 5.1
 * Mem Model: Code is memory model independent
 * Date     : October, 1988
 * Switches : Define TEST to get a test driver
 *
 * Source code is (c) John Rex, 1988.
 * Object code may be used freely.
 *
 * Usage: consider this code fragment as program.c --
 *
 * main(int argc, char **argv)
 * {
 *    void envcmd(int *, char ***, char *);
 *
 *    envcmd(&argc, &argv, "VAR"); \* get cmds from "VAR" *\
 *                                 \* NOTE: must use upper case for
 *                                          name of environ var! *\
 *  ... continues
 * }
 *
 * If we now execute these commands:
 *  C:>set VAR=123 abc
 *  C:>program -d
 *
 * Then the initial values of argc and argv would have been:
 *  argc=2, argv[0]="program", argv[1]="-d".
 * After the call to envcmd(), they would be:
 *  argc=4, argv[0]="program", argv[1]="123", argv[2]="abc", argv[4]="-d"
 *
 **********************************************************************/

#define TEST  /* include test driver */

#include <stdio.h>
#include <stdlib.h> /* for getenv() */
#include <string.h> /* for strtok() */

#define BLEN 200	/* work area - should be at least 128 */
#define DELIM " \t" /* delimiters for parsing args from environ var */

void envcmd(int *old_argc, char ***old_argv, char *varname)
{
	char buffer[BLEN], *env_var, *s, *copy;
	char **new_argv;
	int new_argc;
	int count, i, j;

	env_var = getenv(varname); /* Does environ variable exist? */
	if (env_var == NULL)
		return;	/* exit without changing anything if not */

	strcpy(buffer, env_var); /* get a copy we can play with */

	/* now count the number of arguments in it */
	count = 0;
	if (strtok(buffer, DELIM) != NULL) {
		count++;
		while (strtok(NULL, DELIM) != NULL)
			count++;
	}

	if (count == 0)
		return;	/* exit without changing anything if it was empty */

	/* allocate new argv array */
	new_argc = *old_argc + count;
	new_argv = (char **) malloc(sizeof(char *) * (new_argc + 1));

	/* copy data */
	new_argv[0] = (*old_argv)[0];	/* just copy pointer to prog name */

	copy = strdup(env_var);  /* get a permanent copy of environ var */
	i = 1;
	s = strtok(copy, DELIM); /* and chop it up */
	new_argv[i++] = s;  	 /* first argument */
	while((s = strtok(NULL, DELIM)) != NULL) /* and trailing things */
		new_argv[i++] = s;

	for (j=0; j < *old_argc; j++) /* and copy over real command line args */
		new_argv[i++] = (*old_argv)[j+1];
		/* note that we actually copy one "extra" argument--argv[argc] is
		   defined as NULL, and we must copy it, too. */

	/* finally, reset values for caller */
	*old_argc = new_argc;
	*old_argv = new_argv;
}

#if defined(TEST)

void main(int argc, char **argv)
{

	int i;

	if (argc < 2)
		envcmd(&argc, &argv, "DEFAULT");
	else
		envcmd(&argc, &argv, argv[1]);

	for (i=0; i<argc; i++)
		printf("argv[%d]: %s\n", i, argv[i]);
}
#endif
