/*  
    Header.c  by Ken Sawyer (403) 439-8171
    (C) Copyright Sequiter Software Inc., 1988.  All rights reserved.

    Display dBASE III, or dBASE III PLUS header file information.

    Tested with Quick C 1.0 and Turbo C 1.5

    A Royalty Free License to use this software, complete with
    diskette, is available for $3.00 shipping and handling:

       Sequiter Software Inc.  (403) 439-8171
       P.O. Box 5659, Station L, Edmonton, Alberta, Canada, T6C 4G1

    Note:  This program is adopted from the 'd3use' routine of
           Code Base, Sequiter's dBASE compatible C library.
*/

#ifdef NEVER
#ifdef QUICKC
   #include <sys\types.h>
#endif
#include <sys\stat.h>
#endif

#ifdef NEVER
#ifdef QUICKC
   #include <malloc.h>
#endif
#endif



#include <stdio.h>      /* For the 'lseek' routine */
#include <stdlib.h>     /* For the 'malloc' routine */
#include <io.h>
#include <fcntl.h>      /* These Include Files are for the 'open' routine */

/* This is the structure of the first 10 bytes of the database file */
typedef struct
{
   char  version ;             /* 3 or 0x83 if there is a memo file */
   char  yy ;                  /* Year Last Updated */
   char  mm ;                  /* Month Last Updated (1-12) */
   char  dd ;                  /* The Day Last Updated (1-31) */
   long  records ;             /* The number of records in the database */
   unsigned  header_length ;   /* The length of the header information */
}  BASE_INFO ;

/* This is the structure of the field data within the database header */
typedef  struct
{
   char  name[11] ;            /* Field Name */
   char  type ;                /* Field Type */
   char  filler1[4] ;          /* Reserved (Undefined) */
   char  width ;               /* Field Width */
   char  decimals ;            /* The Number of Decimals */
   char  filler2[14] ;         /* Reserved (Undefined) */
}  FIELD ;

/* Data for Converting the Month to Character Format */
char *month_data[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
                       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" } ;


int main( int argc, char **argv ) 
{
   int        hand ;            /* The File Handle */
   int        i ;               /* Number of Fields Counter */
   int        total_width ;     /* For Totaling the Field Widths */
   BASE_INFO  base_info ;       /* Storage for the first 10 Bytes */
   FIELD     *field_data ;      /* A pointer to memory which 
                                   will be allocated once the 
                                   amount needed is known */

   printf( "dBASE III PLUS Header Display\n" ) ;
   printf( "(C) Copyright Sequiter Software Inc, 1988\n\n") ;

   /* The first argument does not count as it is the name of the program */
   if ( argc < 2 ) 
   {
      printf( "USAGE:  HEADER  DBASE_FILE_NAME\n" ) ;
      exit(1) ;
   }

   /* Open in binary mode so that the read routine does no data interpretation */
   hand =  open( argv[1], O_BINARY | O_RDONLY ) ;
   if ( hand < 0 ) 
   {
      printf( "Error Opening File: %s\n", argv[1] ) ;
      exit(2) ;
   }

   /* Position to the beginning of the file (ie Byte 0L) */
   lseek( hand, 0L, SEEK_SET ) ;

   /* Read in some preliminary header information */
   if ( read( hand, (char *) &base_info, (unsigned int) sizeof(base_info)) 
        !=  (int) sizeof(base_info) ) 
   {
      printf( "Error Reading File:  %s\n", argv[1] ) ;
      exit(3) ;
   }

   /* Do some minor checks on the header information */
   if (base_info.header_length > 0x7FFF ||
      (base_info.version != (char) 3  && base_info.version != (char) 0x83))
   {
      printf( "Not a dBase File:  %s\n", argv[1] ) ;
      exit(4) ;
   }

   printf( "FILE:         %s\n", argv[1] ) ;
   printf( "LAST UPDATED: %s %d, 19%d\n", month_data[base_info.mm-1],
                                          base_info.dd, base_info.yy ) ;
   printf( "RECORDS:      %ld\n", base_info.records ) ;

   printf( "MEMO FILE:    ") ;

   if ( base_info.version == (char) 0x83 ) 
      printf( "PRESENT\n" ) ;
   else
      printf( "NOT PRESENT\n" ) ;

   /* Allocate memory for the complete Header */
   field_data =  (FIELD *) malloc( (size_t) base_info.header_length ) ;
   if ( field_data == (FIELD *) 0 ) 
   {
      printf( "Not Enough Memory\n" ) ;
      exit(5) ;
   }

   /* Read in the whole Header */
   lseek( hand, 0L, SEEK_SET ) ;
   if ( read( hand, (char *) field_data, base_info.header_length )
        !=  (int) base_info.header_length ) 
   {
      printf( "Error Reading File:  %s\n", argv[1] ) ;
      exit(6) ;
   }

   printf( "\nFIELD NAME    TYPE    WIDTH    DECIMALS\n\n" ) ;

   /* Loop until the first byte of the field name is '0xD'.  This
      signals that there are no more fields.  The header length
      check is for some extra protection.
   */
   for ( i=1, total_width= 0 ; field_data[i].name[0] != 0xD && 
              (i+1)*32 <= base_info.header_length; i++ )
   {
      /* Print the Field Data */
      printf( "%-10s     %c%9d%11d\n", field_data[i].name,
				       field_data[i].type,
				 (int) field_data[i].width,
				 (int) field_data[i].decimals ) ;
      total_width +=  field_data[i].width ;  /* Sum the Field Widths */
   }

   /* After the loop 'i' is one more than the total number of fields. 
      The total record width is one more than the field widths.  This
      is because the first byte of the record is a '*' if the record
      is marked for deletion.  Otherwise, the first byte is blank.  */

   printf( "\nNUMBER OF FIELDS: %d   RECORD WIDTH %d\n", i-1, total_width+1) ;

   free( field_data ) ;  /* Free the allocated memory */
   exit(0) ;
}
