/***********************************************************************
 * LJDJGRAF.C - graphics on the HP LaserJet and DeskJet
 *
 * Author: David P. Babcock
 * Date: June 1, 1989
 *
 * Compilers: Turbo C 2.0, MSC 5.1, HP-UX (Unix)
 *    --requires the large memory model
 *
 * A large part of the function g_ellipse() originally appeared in the Weber
 * article cited in the bibliography.
 *
 * This code may be used freely for any purpose so long as authorship      
 *    is acknowledged and that such release does not conflict with other
 *    copyrights.
 * Compile time switches:
 *    DESKJET: if == 1, will compile code for DeskJet rather than LaserJet
 ***********************************************************************/

/* #define DESKJET 1 */

#include "includes.h"       /* Multiple includes and test for MS-DOS */


#define DPI                     300
#define EMS_PAGES               ( ( PAGE_ROWS + ROWS_PER_EMS_PAGE - 1 ) / \
                                     ROWS_PER_EMS_PAGE )
#define FLUSH_PAGE              g_line ( 0, 0, 0, 0, -1 )
#define HEIGHT_INCHES           10.5
#define IROUND(x)               ((int)ROUND(x))
#define MAX(a,b)                (((a)>(b))?(a):(b))
#define MIN(a,b)                (((a)<(b))?(a):(b))
#define PAGE_ROWS               ((int)( DPI * HEIGHT_INCHES ) )
#define READ_ROW(y,buf)         g_accessRow ( (y), 0, (buf) )
#define ROUND(x)                ((long)floor((x)+0.5))
#define ROWS_PER_EMS_PAGE       ( 16384 / ROW_BYTES )
#define ROW_BYTES               (((int)(DPI * WIDTH_INCHES ) + 7 ) / 8 )
#define SQUARE(x)               ((double)((x))*(double)((x)))
#define WIDTH_INCHES            8
#define WRITE_ROW(y,buf)        g_accessRow ( (y), 1, (buf) )


FILE *fout ;

int i, fillingPoly = 0 ;
int g_EMShandle = -1 ;
int emptyRows = 0 ;
int arrayPtr, xx1, xx2, xx3, xx4, yy1, yy2, yy3, yy4 ;
static int  prev_dy, prev_prev_dy, firstX, firstY, secondX, secondY,
                                   thirdX, thirdY, polyX, polyY ;
static int  first_X, prev_X, prev_Y ;
static char *written_to = NULL ;
static char buf [ ROW_BYTES ] ;

struct point {
    int x, y ;
};

struct point *g_pointArray ;


static int  komp ( a, b )
struct point *a, *b ;
{
    return ( a->y == b->y ? a->x - b->x : a->y - b->y ) ;
}


static void  blacken ( y, x_start, x_end )
int y, x_start, x_end ;

/*
    Blacken the pixels from x_start to x_end in row y.  As
    a special case, a y value of INT_MAX flushes the
    working buffer and initializes an array
    in preparation for the next page.
*/

{
    static char SB[8] = {
        0xFF, 0x7F, 0x3F, 0x1F, 0xF, 7, 3, 1    };
    static char EB[8] = {
        0x80, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0xFE, 0xFF  };
    static int  oldy = INT_MAX ;

    int i, sByte, eByte ;

    if ( x_start > x_end ) {
        i = x_start ;
        x_start = x_end ;
        x_end = i ;
    }

    sByte = x_start / 8 ;
    eByte = x_end / 8 ;

    if ( written_to == NULL ) {
        written_to = calloc ( ( PAGE_ROWS + 7 ) / 8, 1 ) ;
    }

    if ( y != oldy ) /* change in row */ {
        if ( oldy != INT_MAX ) {
            WRITE_ROW ( oldy, buf ) ;

            written_to [ oldy / 8 ] |= ( 1 << (oldy % 8) ) ;
        }

        if ( (y != INT_MAX) && (written_to[ y / 8 ] & ( 1 << (y % 8) )) ) {
            /* recover the row */
            READ_ROW ( y, buf ) ;
        }
        else {
            /* zero the working buffer */
            for ( i = 0 ; i < ROW_BYTES ; ++i )
                buf [ i ] = 0 ;
        }
    }

    if ( sByte == eByte ) {
        buf [ sByte ] |= ( SB [ x_start % 8 ] & EB [ x_end % 8 ] ) ;
    }
    else {
        buf [ sByte ] |= ( SB [ x_start % 8 ] ) ;
        buf [ eByte ] |= ( EB [ x_end % 8 ] ) ;
        for ( i = sByte + 1 ; i < eByte ; ++i )
            buf [ i ] = ~0 ;
    }

    oldy = y ;
}


void g_openPoly()
{
    fillingPoly = 1 ;
    prev_prev_dy = prev_dy = INT_MAX ;
    arrayPtr = 0 ;
}


void g_closePoly()
/*
    This procedure flushes the polygon buffer.
*/

{
    if ( prev_dy != INT_MAX ) {
        if ( polyX != firstX || polyY != firstY ) {
            g_line ( polyX, polyY, firstX, firstY, 0 ) ;
        }
        g_line ( firstX, firstY, secondX, secondY, 0 ) ;
        g_line ( secondX, secondY, thirdX, thirdY, 0 ) ;
        g_fillPoly ( g_pointArray, arrayPtr ) ;
    }
    fillingPoly = 0 ;
}


void g_line ( x1, y1, x2, y2, width )
int x1, y1, x2, y2, width ;
{
    /*
    Draws a fat line.  As a special case, a width of -1 draws
    nothing, but sends each row in the page buffer to the
    output routine.
    */

    int filled, i, y, dx, dy ;
    double  hw, sine, cosine, tangent ;

    if ( width == -1 ) /* flush page */ {
        blacken ( INT_MAX, 0, 0 ) ;

        if ( written_to != NULL ) {
            for ( y = 0 ; y < PAGE_ROWS ; ++y ) {
                if ( written_to [ y / 8 ] & ( 1 << (y % 8) ) ) {
                    READ_ROW ( y, buf ) ;
                    g_outRow ( buf ) ;
                }
                else {
                    g_outEmptyRow ( ) ;
                }
            }
            free ( written_to ) ;
            written_to = NULL ;
        }
    }
    else if ( fillingPoly ) {
        dy = ( y2 > y1 ? -1 : ( y2 < y1 ? 1 : 0 ) ) ;
        polyX = x2 ;
        polyY = y2 ;
        if ( prev_dy == INT_MAX ) {
            firstX = x1 ;
            firstY = y1 ;
            secondX = x2 ;
            secondY = y2 ;
        }
        else if ( prev_prev_dy == INT_MAX ) {
            if ( prev_dy == 0 && dy == 0 ) {
                secondX = x2 ;
                secondY = y2 ;
                goto g_lineret ;
            }
            else {
                thirdX = x2 ;
                thirdY = y2 ;
            }
        }
        else if ( dy != 0 ) {
            bline ( x1, y1, x2, y2, 0, 0 ) ;
            if ( !( prev_dy == 0 && dy == prev_prev_dy ) ) {

            /* the condition fails if and only if we have a non-zero dy
            following one or more zero dy's following a dy of the same
            sign as the current one.  In this case alone, it is an error
            to write (x1,y1), as to do so would have the effect of writing both
            end points of the zero-dy line--giving an odd (i.e. wrong) number
            of points on the raster line for the closed polygon.  The fact
            that we separately trace the edge of the polygon means that
            it doesn't matter which end point we ignore; any error here,
            which can only be a failure to write the zero-dy segment,
            is exactly covered by the edge trace. */

                riteDot ( x1, y1 ) ;
            }
            if ( dy + prev_dy == 0 ) {
                /* grazing, so we need to write this point twice */
                riteDot ( x1, y1 ) ;
            }
        }
        else {
            if ( prev_dy != 0 )
                riteDot ( x1, y1 ) ;
            else
                goto g_lineret ;
            /* so as to leave prev's as they were--this takes
                care of the "...or more..." in the preceding comment */
        }
        prev_prev_dy = prev_dy ;
        prev_dy = dy ;
    }
    else {
        arrayPtr = 0 ;

        hw = ( double ) width / 2.0 - 0.5 ; /* amount to add on either
          side of line to get desired width. The 0.5 is because we assume
          that the line itself is 1 pixel wide, which is roughly true
                      on a LaserJet or DeskJet */

        /* y2 must be >= y1 */

        if ( y2 < y1 ) {

            i = x1 ;
            x1 = x2 ;
            x2 = i ;

            i = y1 ;
            y1 = y2 ;
            y2 = i ;

        }

        dy = y2 - y1 ;
        dx = x2 - x1 ;

        if ( dx == 0 ) /* vertical line */ {
            if ( width )
                addRule ( x1 - width / 2, y1, x2 + ( width - 1 ) / 2, y2 ) ;
        }
        else if ( dy == 0 ) /* horizontal line */ {
            if ( width )
                addRule ( x1, y1 - width / 2, x2, y2 + ( width - 1 ) / 2 ) ;
        }
        else {
            tangent = ( ( double ) ( y2 - y1 ) /
                        ( double ) ( x2 - x1 ) ); /* by definition */
            sine = tangent /
                sqrt ( 1.0 + tangent * tangent ); /* standard identity */
            if ( y2 < y1 ) sine = -sine ;
            cosine = sine / tangent ;

            xx1 = x1 + ( ROUND ( ( double ) hw * sine ) )  ;
            xx2 = x2 + ( ROUND ( ( double ) hw * sine ) )  ;
            xx3 = x2 - ( ROUND ( ( double ) hw * sine ) )  ;
            xx4 = x1 - ( ROUND ( ( double ) hw * sine ) )  ;

            yy1 = y1 - ( ROUND ( ( double ) hw * cosine ) )  ;
            yy2 = y2 - ( ROUND ( ( double ) hw * cosine ) )  ;
            yy3 = y2 + ( ROUND ( ( double ) hw * cosine ) )  ;
            yy4 = y1 + ( ROUND ( ( double ) hw * cosine ) )  ;

            filled = 0 ;

            /*  In the following, care is taken with crossing
                vs. grazing vertices, just as with polygons.
                That is the reason for the two booleans passed
                to bline() */

            if ( yy2 == yy3 ) {
                if ( xx2 == xx3 ) {
                    filled = 1 ;
                    fill_line ( xx1, yy1, xx2, yy2 ) ;
                }
                else {
                    bline ( xx1, yy1, xx2, yy2, 1, 1 ) ;
                    bline ( xx4, yy4, xx3, yy3, 1, 1 ) ;
                }
            }
            else if ( yy3 == yy4 ) {
                if ( xx3 == xx4 ) {
                    filled = 1 ;
                    fill_line ( xx2, yy2, xx3, yy3 ) ;
                }
                else {
                    bline ( xx2, yy2, xx3, yy3, 1, 1 ) ;
                    bline ( xx1, yy1, xx4, yy4, 1, 1 ) ;
                }
            }
            else {
                bline ( xx1, yy1, xx2, yy2, 1, 1 ) ;
                bline ( xx2, yy2, xx3, yy3, 0, 1 ) ;

                bline ( xx1, yy1, xx4, yy4, 1, 1 ) ;
                bline ( xx4, yy4, xx3, yy3, 0, 1 ) ;
            }

            if ( !filled )
                g_fillPoly ( g_pointArray, arrayPtr ) ;

        }
    }
    g_lineret:
    return ;

}


void addRule ( x_start, y_start, x_end, y_end )
int x_start, y_start, x_end, y_end ;

/*
    This blackens the area specified by a rule (in other
    words a rectangle) by getting the parameters in the
    relation expected by blacken() and then calling that
    function.
*/

{
    int a, b, y ;

    if ( y_start <= y_end ) {
        a = y_start ;
        b = y_end ;
    }
    else {
        a = y_end ;
        b = y_start ;
    }
    for ( y = a; y <= b; ++y ) {
        blacken ( y, x_start, x_end ) ;
    }
}


void g_fillPoly ( g_pointArray, arrayPtr )
struct point *g_pointArray ;
int arrayPtr ;

/*
    This sorts the array of points and fills each consecutive pair.
*/
{

    int i ;

    ( void ) qsort ( g_pointArray, arrayPtr, sizeof ( struct point ), komp ) ;

    /* for each y, set the appropriate line segment or segments */

    for ( i = 0 ; i < arrayPtr ; i += 2 ) {

        addRule ( g_pointArray[i].x , g_pointArray[i].y , g_pointArray[i+1].x,
                  g_pointArray[i+1].y );
    }
}


static void  riteDot ( x, y )
int x, y ;
{
    g_pointArray [ arrayPtr ].x = x ;
    g_pointArray [ arrayPtr++ ].y = y ;

}

static void bline(x1, y1, x2, y2, botpix, toppix)
int x1, y1, x2, y2, botpix, toppix;
{
    int x, y, endY, startY ;
    long int    x2mx1 ;
    int y2my1, hy2my1 ;

    if ( y2 == y1 ) {
        if ( botpix && toppix ) {
            riteDot ( x1, y1 ) ;
            if ( x1 != x2 )
                riteDot ( x2, y2 ) ;
        }
        return;
    }

    if ( y2 < y1 ) {

        i = x1 ;
        x1 = x2 ;
        x2 = i ;

        i = y1 ;
        y1 = y2 ;
        y2 = i ;

    }

    startY = botpix ? y1 : y1 + 1 ;
    endY = toppix ? y2 : y2 - 1 ;

    y2my1 = y2 - y1 ;

    /* This being integer arithmetic, rounding in division
        is best handled by adding or subtracting half the divisor
        to/from the dividend in the first place.  The variable
        hy2my1 will do that. */

    hy2my1 = ( x2 < x1 ) ? -y2my1 / 2 : y2my1 / 2 ;
    x2mx1 = ( long ) x2 - x1 ;

    for ( y = startY ; y <= endY ; ++y ) {
        x = ( ( long ) ( y - y1 ) * ( x2mx1 ) + hy2my1 )  / y2my1 + x1 ;
        riteDot ( x , y ) ;
    }
}


static void  fill_line(x1, y1, x2, y2)
int x1, y1, x2, y2 ;
{

    /* This is an implementation of Bresenham's line drawing
        algorithm.  Explanations are widely available in the
        literature.  */

    int inc1, inc2, inc3, xend, yend, d, x, y, dx, dy ;
    prev_Y = ~0 ;

    dx = abs(x2 - x1);
    dy = abs(y2 - y1);

    if (dy <= dx) {
        if (x1 > x2) {
            x = x2;
            y = y2;
            xend = x1;
            dy = y1 - y2;
        }
        else {
            x = x1;
            y = y1;
            xend = x2;
            dy = y2 - y1;
        }
        inc1 = dy << 1;
        inc2 = (dy - dx) << 1;
        inc3 = (dy + dx) << 1;
        d = (dy >= 0) ? inc1 - dx : inc1 + dx ;
        while (x < xend) {
            rite(x, y);
            /* plot point */
            x++;
            if (d >= 0) {
                if (dy <= 0)
                    d += inc1 ;
                else {
                    y++;
                    d += inc2 ;
                }
            }
            else {
                if (dy >= 0)
                    d += inc1;
                else {
                    y--;
                    d += inc3 ;
                }
            }
        }
    }
    else {
        if (y1 > y2) {
            y = y2;
            x = x2;
            yend = y1;
            dx = x1 - x2;
        }
        else {
            y = y1;
            x = x1;
            yend = y2;
            dx = x2 - x1;
        }
        inc1 = dx << 1;
        inc2 = (dx - dy) << 1;
        inc3 = (dx + dy) << 1;
        d = (dx >= 0) ? inc1 - dy : inc1 + dy ;
        while (y < yend) {
            rite(x, y);
            /* plot point */
            y++;
            if (d >= 0) {
                if (dx <= 0)
                    d += inc1;
                else {
                    x++;
                    d += inc2;
                }
            }
            else {
                if (dx >= 0)
                    d += inc1;
                else {
                    x--;
                    d += inc3;
                }
            }
        }
    }
    rite(x, y);
    flush() ;
    /* plot point */
}


static void rite ( x, y )
int x, y ;
{
    if ( y != prev_Y ) {
        if ( prev_Y != ~0 )
            addRule ( first_X, prev_Y, prev_X, prev_Y ) ;
        first_X = x ;
        prev_Y = y ;
    }
    prev_X = x ;
}


static void flush ( )
{
    addRule ( first_X, prev_Y, prev_X, prev_Y ) ;
}



#if defined(__MSDOS__) || defined(MSDOS)

int g_getEMShandle ( pages )
int pages ;
{
    union REGS regs ;

    regs.h.ah = 0x43;
    regs.x.bx = pages;
    int86 ( 0x67, &regs, &regs );
    return (  regs.h.ah ? -1 : regs.x.dx ) ;
}


unsigned int g_getEMSframe ( void )
{
    union REGS regs ;

    regs.h.ah = 0x41;
    int86 ( 0x67, &regs, &regs );
    return ( regs.x.bx );
}


int g_mapEMSpage ( page )
int page ;
{
    union REGS regs ;

    regs.x.ax = 0x4400;
    regs.x.bx = page;
    regs.x.dx = g_EMShandle;
    int86 ( 0x67, &regs, &regs );
    return ( regs.h.ah ? -1 : 0 );
}


int g_freeEMShandle ( handle )
int handle ;
{
    union REGS regs ;

    regs.h.ah = 0x45;
    regs.x.dx = handle;
    int86 ( 0x67, &regs, &regs );
    return ( regs.h.ah ? -1 : 0 ) ;
}

#endif

void g_accessRow ( y, write, buf )
int y, write ;
char    *buf ;
{
    char    *base ;
    int page16, offset ;

    page16 = y / ROWS_PER_EMS_PAGE ;
    offset = ( y % ROWS_PER_EMS_PAGE ) * ROW_BYTES ;

#if defined(__MSDOS__) || defined(MSDOS)
    base = g_EMSframe ;
    g_mapEMSpage ( page16 ) ;
#else
    base = g_base [ page16 ] ;
#endif

    if ( write ) {
        memcpy ( base + offset, buf, ROW_BYTES ) ;
    }
    else {
        memcpy ( buf, base + offset, ROW_BYTES ) ;
    }
}


int g_init16 ( )
{
    unsigned int EMSsegment;

#if defined(__MSDOS__) || defined(MSDOS)
    g_EMShandle = g_getEMShandle ( EMS_PAGES ) ;
    EMSsegment = g_getEMSframe();
    g_EMSframe = MK_FP(EMSsegment, 0U);

    return ( g_EMShandle != -1 && g_EMSframe != NULL ? 0 : -1 ) ;
#else
    int i;

    for ( i = 0 ; i < EMS_PAGES ; ++i ) {
        g_base [ i ] = calloc ( 1, 16384 ) ;
    }
    return ( g_base [ EMS_PAGES - 1 ] != NULL ? 0 : -1 ) ;
#endif
}


void g_outRow ( buf )
char    *buf ;
{
    int i, bytes ;
#if DESKJET
    char    tbuf [ ROW_BYTES ] ;
#endif

    if ( emptyRows ) {
        fprintf ( fout, "\033*rB\033*p+%dY", emptyRows ) ;
        emptyRows = 0 ;
#if DESKJET
        fprintf ( fout, "\033*r1A\033*b2M" ) ;
#else

        fprintf ( fout, "\033*t300R\033*r1A" ) ;
#endif
    }

    for ( i = ROW_BYTES - 1 ; i >= 0 ; --i ) {
        if ( buf [ i ] != 0 )
            break ;
    }
    bytes = i + 1 ;
#if DESKJET
    for ( i = 0 ; i < bytes ; ++i )
        tbuf [ i ] = buf [ i ] ;
    bytes = compress ( buf, tbuf, bytes ) ;
#endif

    fprintf ( fout, "\033*b%dW", bytes ) ;
    for ( i = 0 ; i < bytes ; ++i ) {
        fputc ( buf [ i ], fout ) ;
    }
}

void g_outEmptyRow ( )
{
    ++emptyRows ;
}

static int  compress ( outRow, inRow, bytes ) /* for DeskJet only */
char    *outRow, *inRow ;
int bytes ;
{

    /* returns the number of bytes in the compressed row */
    /* we implement what the DeskJet manual calls
        "compaction graphics mode 2" */

    int i, j, k, rep, dif ;

    j = rep = dif = 0 ;

    for ( i = 1 ; i < bytes ; ++i ) {

        if ( inRow [ i ] == inRow [ i - 1 ] ) {
            if ( dif ) {
                outRow [ j++ ] = dif - 1 ;
                for ( k = i - dif - 1 ; k < i - 1 ; ++k )
                    outRow [ j++ ] = inRow [ k ] ;
                dif = 0 ;
                rep = 1 ;
            }
            else {
                if ( ++rep == 127 ) {
                    outRow [ j++ ] = -rep ;
                    outRow [ j++ ] = inRow [ i - 1 ] ;
                    rep = 0 ;
                    if ( ++i == bytes - 1 ) {
                        outRow [ j++ ] = 0 ;
                        outRow [ j++ ] = inRow [ bytes - 1 ] ;
                    }
                }
            }
        }
        else {
            if ( rep ) {
                outRow [ j++ ] = -rep ;
                outRow [ j++ ] = inRow [ i - 1 ] ;
                rep = 0 ;
                if ( i == bytes - 1 ) {
                    outRow [ j++ ] = 0 ;
                    outRow [ j++ ] = inRow [ bytes - 1 ] ;
                    ++i ;
                }
            }
            else {
                if ( ++dif == 127 ) {
                    outRow [ j++ ] = dif ;
                    for ( k = i - dif ; k <= i ; ++k )
                        outRow [ j++ ] = inRow [ k ] ;
                    dif = 0 ;
                    if ( ++i == bytes - 1 ) {
                        outRow [ j++ ] = 0 ;
                        outRow [ j++ ] = inRow [ bytes - 1 ] ;
                    }
                }
            }
        }
    }

    if ( rep ) {
        outRow [ j++ ] = -rep ;
        outRow [ j++ ] = inRow [ i - 1 ] ;
    }
    else if ( dif ) {
        outRow [ j++ ] = dif ;
        for ( k = i - dif - 1 ; k < i ; ++k )
            outRow [ j++ ] = inRow [ k ] ;
    }

    return ( j ) ;

}


void g_ellipse ( x0, y0, x1, y1, x2, y2 )
int x0, y0, x1, y1, x2, y2 ;

{
    double  xp2, yp2, maxabs, scale ;
    double  cosine, sine, cr1, cr2alt, m ;
    double  aa, bb, cc, dd, ee ;
    double  oneovrcr2altsqr, x0mx1, x2mx1, y2my1, y0my1, oneovrcr1;
    double  temp1, temp2 ;
    long int    pa, pb, pc, pd, pe ;
    long int    k1, k2, k3, a, b, d, w;
    int xpt, ypt, octant;
    long int    x, y, askip, bskip;

    x0mx1 = ( double ) ( x0 - x1 ) ;
    x2mx1 = ( double ) ( x2 - x1 ) ;
    y0my1 = ( double ) ( y0 - y1 ) ;
    y2my1 = ( double ) ( y2 - y1 ) ;

    oneovrcr1 = 1.0 / ( cr1 = sqrt(SQUARE(x0mx1) + SQUARE(y0my1))) ;

    /* translate so that (x1,y1) is at (0,0) and y0 is 0 */

    xp2 = ( x0mx1 * x2mx1 + y0my1 * y2my1 ) * oneovrcr1 ;
    yp2 = (-y0my1 * x2mx1 + x0mx1 * y2my1 ) * oneovrcr1 ;

    cr2alt = yp2 ;

    if ( cr2alt != 0.0 ) { /* if 0.0, we have co-linear points */
        oneovrcr2altsqr = 1.0 / SQUARE(cr2alt) ;
        cosine = x0mx1 * oneovrcr1 ;
        sine   = y0my1 * oneovrcr1 ;

        m = ( cr1 - xp2 ) / yp2 ;

        temp1 = ( cosine - m * sine ) * oneovrcr1 ;
        temp2 = ( sine + m * cosine ) * oneovrcr1 ;

        dd = -2 * temp1 ;
        ee = -2 * temp2 ;
        aa = SQUARE ( temp1 ) + SQUARE ( sine ) * oneovrcr2altsqr ;
        bb = 2 * ( temp1 * temp2 - sine * cosine * oneovrcr2altsqr ) ;
        cc = SQUARE ( temp2 ) + SQUARE ( cosine ) * oneovrcr2altsqr ;

        /* scale aa,bb,cc,dd,ee to fit range */

        maxabs = fabs(aa);
        if (fabs(bb)>maxabs)
            maxabs = fabs(bb);
        if (fabs(cc)>maxabs)
            maxabs = fabs(cc);
        if (fabs(dd)>maxabs)
            maxabs = fabs(dd);
        if (fabs(ee)>maxabs)
            maxabs = fabs(ee);

        scale = fabs(( 0x20000000 - 2  ) / maxabs );

        pa  = ROUND ( aa * scale ) ;
        pb  = ROUND ( bb * scale ) ;
        pc  = ROUND ( cc * scale ) ;
        pd  = ROUND ( dd * scale ) ;
        pe  = ROUND ( ee * scale ) ;
        xpt = x1 ;
        ypt = y1 ;

        x = y = 0L;          /* start from origin */
        k1 = 2 * pa;        /* find initial octant, try first octant */
        k2 = k1 + pb;
        k3 = k2 + 2 * pc + pb;
        b = -pd - pa - pb / 2;
        a = -b + pe;
        d = b - pe / 2 - pc / 4;
        octant = 1;
        if ((a < 0) || (b < 0)) {                   /* try second octant */
            k1 = -2 * pc;
            k2 = k1 - pb;
            k3 = k2 - 2 * pa - pb;
            b = pe + pc + pb / 2;
            a = -b - pd;
            d = b + pd / 2 + pa / 4;
            octant = 2;
        }
        if ((a < 0) || (b < 0)) {                   /* try third octant */
            k1 = 2 * pc;
            k2 = k1 - pb;
            k3 = k2 + 2 * pa - pb;
            b = -pe - pc + pb / 2;
            a = -b - pd;
            d = b + pd / 2 - pa / 4;
            octant = 3;
        }
        if ((a < 0) || (b < 0)) {                   /* try fourth octant */
            k1 = -2 * pa;
            k2 = k1 + pb;
            k3 = k2 - 2 * pc + pb;
            b = -pd + pa - pb / 2;
            a = -b - pe;
            d = b + pe / 2 + pc / 4;
            octant = 4;
        }
        if ((a < 0) || (b < 0)) {                   /* try fifth octant */
            k1 = 2 * pa;
            k2 = k1 + pb;
            k3 = k2 + 2 * pc + pb;
            b = pd - pa - pb / 2;
            a = -b - pe;
            d = b + pe / 2 - pc / 4;
            octant = 5;
        }
        if ((a < 0) || (b < 0)) {                   /* try sixth octant */
            k1 = -2 * pc;
            k2 = k1 - pb;
            k3 = k2 - 2 * pa - pb;
            b = -pe + pc + pb / 2;
            a = -b + pd;
            d = b - pd / 2 + pa / 4;
            octant = 6;
        }
        if ((a < 0) || (b < 0)) {                   /* try seventh octant */
            k1 = 2 * pc;
            k2 = k1 - pb;
            k3 = k2 + 2 * pa - pb;
            b = pe - pc + pb / 2;
            a = -b + pd;
            d = b - pd / 2 - pa / 4;
            octant = 7;
        }
        if ((a < 0) || (b < 0)) {                   /* try eighth octant */
            k1 = -2 * pa;
            k2 = k1 + pb;
            k3 = k2 - 2 * pc + pb;
            b = pd + pa - pb / 2;
            a = -b + pe;
            d = b - pe / 2 + pc / 4;
            octant = 8;
        }
        /* start = octant; */         /* starting octant is ending octant */
        askip = bskip = 0;            /* smooth gradients at first */
        for (; ; )                    /* for each point in quadrant */
        {                             /* draw */
            gdot((int)x + xpt, (int)y + ypt);
            if (d < 0) {              /* move horizontal or vertical */
                switch (octant) {
                    case 1 :
                    case 8 :
                        x++;
                        break;
                    case 2 :
                    case 3 :
                        y++;
                        break;
                    case 4 :
                    case 5 :
                        x--;
                        break;
                    case 6 :
                    case 7 :
                        y--;
                        break;
                }
                b -= k1;
                a += k2;
                d += b;
            }
            else {                       /* move diagonal */
                switch (octant) {
                    case 1 :
                    case 2 :
                        x++;
                        y++;
                        break;
                    case 3 :
                    case 4 :
                        x--;
                        y++;
                        break;
                    case 5 :
                    case 6 :
                        x--;
                        y--;
                        break;
                    case 7 :
                    case 8 :
                        x++;
                        y--;
                        break;
                }
                b -= k2;
                a += k3;
                d -= a;
            }

            /* Weber's terminating condition was wrong; the starting octant
    might be left incomplete.  The following line is the substituted
    termination condition. */

            if ( x == 0L && y == 0L )
                return ;
            if (b < 0)              /* square octant change */ {
                if (bskip == 0)
                    (octant == 1) ? (octant = 8) : (octant--);
                /* if (octant == start) return; */
                w = k2 - k1;
                k1 = -k1;
                k2 = w + k1;
                k3 = 4 * w - k3;
                b = -b - w;
                d = b - a - d;
                a = a - 2 * b - w;
                askip = bskip = 0;
                if (b < 0)
                    bskip = 1;   /* steep gradients */
            }
            if (a < 0)              /* diagonal octant change  */ {
                if (askip == 0)
                    (octant == 1) ? (octant = 8) : (octant--);
                /* if (octant == start) return; */
                w = 2 * k2 - k3;
                k1 = w - k1;
                k2 = k2 - k3;
                k3 = -k3;
                b = b + a - k2 / 2;
                d = b - a / 2 - d + k3 / 8;
                a = w / 2 - a;
                askip = bskip = 0;
                if (a < 0)
                    askip = 1;   /* steep gradients */
            }
        }
    }
}


static void gdot (x, y)
int x, y ;
{
    blacken ( y, x, x ) ;
}


void g_bezier ( _x1, _y1, _x2, _y2, _xc1, _yc1, _xc2, _yc2 )
int _x1, _y1, _x2, _y2, _xc1, _yc1, _xc2, _yc2 ;
{
    _bezier ( ( long ) _x1 << 16, ( long ) _y1 << 16,
              ( long ) _x2 << 16, ( long ) _y2 << 16,
              ( long ) _xc1 <<16, ( long ) _yc1 << 16,
              ( long ) _xc2 << 16, ( long ) _yc2 << 16 ) ;

    /* We expand to longs because that gives much better accuracy
        and avoids problems with the terminating condition */
}

static void _bezier ( x1, y1, x2, y2, xc1, yc1, xc2, yc2 )
long    x1, y1, x2, y2, xc1, yc1, xc2, yc2 ;
{
    long    xm1, ym1, xm2, ym2, xm3, ym3, xm4, ym4, xm5, ym5, xm6, ym6 ;

    xm1 = ( x1 + xc1 ) / 2 ;
    ym1 = ( y1 + yc1 ) / 2 ;

    xm2 = ( x2 + xc2 ) / 2 ;
    ym2 = ( y2 + yc2 ) / 2 ;

    xm3 = ( xc1 + xc2 ) / 2 ;
    ym3 = ( yc1 + yc2 ) / 2 ;

    xm4 = ( xm1 + xm3 ) / 2 ;
    ym4 = ( ym1 + ym3 ) / 2 ;

    xm5 = ( xm2 + xm3 ) / 2 ;
    ym5 = ( ym2 + ym3 ) / 2 ;

    xm6 = ( xm4 + xm5 ) / 2 ;
    ym6 = ( ym4 + ym5 ) / 2 ;

    if ( labs ( ( (x1 + 32768) >> 16 ) - ( (x2 + 32768) >> 16 ) ) > 1L ||
        labs ( ( (y1 + 32768) >> 16 ) - ( (y2 + 32768) >> 16 ) ) > 1L ) {
        _bezier ( x1, y1, xm6, ym6, xm1, ym1, xm4, ym4 ) ;
        _bezier ( xm6, ym6, x2, y2, xm5, ym5, xm2, ym2 ) ;
    }
    else   {
        plotxy ( (int) ( (x1 + 32768) >> 16 ), (int) ( (y1 + 32768) >> 16 ) ) ;
        plotxy ( (int) ( (x2 + 32768) >> 16 ), (int) ( (y2 + 32768) >> 16 ) ) ;
    }
}


static void plotxy ( x, y )
int x, y ;
{
    blacken ( y, x, x ) ;
}

int g_clip(x1Int, y1Int, x2Int, y2Int, x1, y1, x2, y2, XL, YB, XR, YT)
int *x1Int, *y1Int, *x2Int, *y2Int, x1, y1, x2, y2, XL, YB, XR, YT ;
{

    /* The first four arguments are pointers to the return values; the next
    four are the end points of the line to be clipped; the last four are
    the corners of the clipping window.

    We view the world as a 3x3 grid as follows:

                       -            xx1 and xx2            +

                             0           1           2
                        ___________________________________
             +         |           |           |           |
                    2  |           |           |           |
                       |___________|___________|___________|
            yy1        |           | clipping  |           |
            and     1  |           |  window   |           |
            yy2        |___________|___________|___________|
                       |           |           |           |
                    0  |           |           |           |
             -         |___________|___________|___________|

    The variables _xx1, _yy1, _xx2, and _yy2 all take values of 0, 1, or 2
    depending on the value of the x and y coordinates with respect to the
    clip window.  All the logic that follows makes use of this.

    */

    int _xx1, _yy1, _xx2, _yy2, insides, inside1;
    int x, y, _Xx, _Yy, xInt, yInt, deltaYgreater,
              xLow, xHigh, yLow, yHigh, found;

    _xx1 = x1 < XL   ? 0 : 1 + (x1 > XR);
    _xx2 = x2 < XL   ? 0 : 1 + (x2 > XR);
    _yy1 = y1 < YB   ? 0 : 1 + (y1 > YT);
    _yy2 = y2 < YB   ? 0 : 1 + (y2 > YT);

    /* determine whether 0, 1, or both of the end points
       are inside the clipping window. */

    insides = (inside1 = (_xx1 == 1) && (_yy1 == 1)) +
        ((_xx2 == 1) && (_yy2 == 1)) ;

    if (insides == 2) { /* both points are inside:
                           the clipped line is the original line */
        *x1Int = x1;
        *y1Int = y1;
        *x2Int = x2;
        *y2Int = y2;
        return(1);
    }

    if ( ( found = insides ) != 0 ) { /* one point is inside:
                            that will be one end of the clipped line */
        *x1Int = inside1 ? x1 : x2 ;
        *y1Int = inside1 ? y1 : y2 ;
    }
    else { /* both points are outside */
        if ( !( ( _xx1 + _xx2 ) & 0xFFFB ) ) /* sum is 0 or 4 */
            /* the line is entirely to the left or
               entirely to the right of the clipping window */
            return ( 0 ) ;

        if ( !( ( _yy1 + _yy2 ) & 0xFFFB ) ) /* sum is 0 or 4 */
            /* the line is entirely above or
               entirely below the clipping window */
            return ( 0 ) ;
    }

    xLow =  _xx1 < _xx2 ? _xx1 : _xx2 ;
    xHigh = _xx1 > _xx2 ? _xx1 : _xx2 ;
    yLow =  _yy1 < _yy2 ? _yy1 : _yy2 ;
    yHigh = _yy1 > _yy2 ? _yy1 : _yy2 ;

    deltaYgreater = ( abs((y1 >> 1) - (y2 >> 1)) >
                      abs((x1 >> 1) - (x2 >> 1)) );
    /* the shifts avoid possible overflow during the subtractions */

    /* at this point we have taken care of the easy cases.  From here on
    we will be looking at where the test line intersects one or another
    window edge. */

    if (deltaYgreater)
        goto doYs;

    doXs:
    for (x = xLow; x < xHigh; x++) {
        if (found == 2)
            return(1);
        _Xx = x ? XR : XL;
        if ( ( _Xx >= x1 ) && ( _Xx <= x2 ) ||
             ( _Xx >= x2 ) && ( _Xx <= x1 ) ) {
            yInt = yIntercept ( _Xx, x1, y1, x2, y2 ) ;
            if ( yInt >= YB && yInt <= YT ) {
                if ( found ) {
                    *x2Int = _Xx ;
                    *y2Int = yInt ;
                }
                else {
                    *x1Int = _Xx ;
                    *y1Int = yInt ;
                }
                ++found ;
            }
        }
    }
    if ( deltaYgreater )
        goto doneXY ;

    doYs:
    for (y = yLow; y < yHigh; ++y) {
        if (found == 2)
            return(1);
        _Yy = y ? YT : YB ;
        if ( ( _Yy >= y1 ) && ( _Yy <= y2 ) ||
             ( _Yy >= y2 ) && ( _Yy <= y1 ) ) {
            xInt = xIntercept ( _Yy, x1, y1, x2, y2 ) ;
            if ( xInt >= XL && xInt <= XR ) {
                if ( found ) {
                    *x2Int = xInt ;
                    *y2Int = _Yy ;
                }
                else {
                    *x1Int = xInt ;
                    *y1Int = _Yy ;
                }
                ++found ;
            }
        }
    }
    if (deltaYgreater)
        goto doXs;

    doneXY:
    return(found == 2);
}

static int xIntercept ( y, x1, y1, x2, y2 )
int y, x1, y1, x2, y2 ;
{
    long  num, den, hden ;

    if ( y1 == y2 ) {
        return ( ( int ) ( ( long ) x1 + x2 ) / 2 ) ;
    }
    else {
        num = ( ( long ) y - y1 ) * ( ( long ) x2 - x1 ) ;
        den = ( long ) y2 - y1 ;
        hden = den / 2 ;
        /* result will be negative if and only if the numerator or the
        denominator, but not both, is negative.  This affects rounding. */

        if ( ((num < 0L) + (den < 0L)) == 1 ) num -= hden ; else num += hden ;
        return ( ( num / den ) + x1 ) ;
    }
}

static int yIntercept ( x, x1, y1, x2, y2 )
int x, x1, y1, x2, y2 ;
{
    long  num, den, hden ;

    if ( x1 == x2 ) {
        return ( ( int ) ( ( long ) y1 + y2 ) / 2 ) ;
    }
    else {
        num = ( ( long ) x - x1 ) * ( ( long ) y2 - y1 ) ;
        den = ( long ) x2 - x1 ;
        hden = den / 2 ;

        /* result will be negative if and only if the numerator
        or the denominator, but not both, is negative.  This
        affects rounding. */
        if ( ( num < 0L ) + ( den < 0L ) == 1 )
             num -= hden ; else num += hden ;
        return ( ( num / den ) + y1 ) ;
    }
}

void ctrlC ( )
{

#if defined(MSDOS) || defined(__MSDOS__)
    g_freeEMShandle ( g_EMShandle ) ;
#endif
    exit ( -1 ) ;
}


void main ()
{

    /* allocate page buffer */
    if ( g_init16 ( ) == 0 ) {

        /* set up Ctrl-C handler */
        signal ( SIGINT, ctrlC ) ;

        /* allocate array of polygon points */
        g_pointArray = ( struct point *)
                         calloc ( 16300, sizeof ( struct point ) );

        /* open output file */
        fout = fopen ( "test.pcl", "wb" ) ;

#if DESKJET

        fprintf ( fout, "\033E\033*r1A\033*b2M" ) ;

#else

        /* Initialize, and set to 300dpi resolution */
        fprintf ( fout, "\033E\033*t300R\033*r1A" ) ;
#endif

        /*
        user code starts
        */

        puts ("Drawing Bezier Curves");
        g_bezier ( 0, 0, 1949, 0, 0, 2700, 10, 2700 ) ;

        puts ("Drawing Ellipse");
        g_ellipse ( 1200, 1500, 1800, 1300, 1150, 300 ) ;

        puts ("Drawing Line");
        g_line ( 100, 100, 1800, 2600, 10 ) ;

        puts ("Drawing Polygon");
        g_openPoly ( ) ;
        g_line ( 100, 2690, 400, 2100, 1 ) ;
        g_line ( 400, 2100, 700, 2690, 1 ) ;
        g_line ( 700, 2690, 10, 2300, 1 ) ;
        g_line ( 10, 2300, 790, 2300, 1 ) ;
        g_line ( 790, 2300, 100, 2690, 1 ) ;
        g_closePoly ( ) ;

        /*
        user code ends
        */

        puts ("Writing file");
        FLUSH_PAGE ;

        fprintf ( fout, "\033*rB\014\033E" ) ;
        fclose ( fout ) ;

#if defined(MSDOS) || defined(__MSDOS__)
        g_freeEMShandle ( g_EMShandle ) ;
#endif
    }
    else fprintf ( stderr, "Unable to initialize page buffer" ) ;

}