/********************** STACKCHK.C **********************
* Author: David Michmerhuizen
* Compilers:  Turbo C V1.5 - use as is
*             MSC 5.1      - use as is
*
* Memory Models: all, even those where DS != SS
*
* Checking new compilers:
*    A test driver is included at the end of stackchk.c.  It
*    should run correctly through subrc() and then report an
*    error in subrd().  Define the preprocessor variable
*    DRIVER to use this code.
*
* Usage:  To setup to use the stack checking feature, you must
*         rename your main() module to real_main().  This file
*         contains a main() that will be called in its place.
*         After the needed stack data is recorded, your
*         real_main() is called.  You then call the test routine
*         as often as desired from within your code:
*
*         Test routine: int check_valid_stack(char *id_string);
*         Arguments: char *id_string -- string identifying caller
*         Returns: TRUE if stack corrupted, FALSE otherwise
*         Example: if (check_valid_stack("end of routine xyz")) exit(1);
*
********************************************************/
#define DRIVER 1    /* compile in the test driver */

#include <dos.h>

#define MAX_STACK_DEPTH  300 /* Fails if > than this # of frame pointers */

static unsigned int far *framepointer, far *stacktop, i;
struct SREGS sregs;

int check_valid_stack( calling_id)
char *calling_id;
{
    unsigned far *stackbottom; /* used for its address */

    i = 0;
    stackbottom  = (unsigned far *) &stackbottom; /* get address of stack */
    framepointer = stackbottom + 2; /* find framepointer */
	segread(&sregs);

    while ( framepointer < stacktop    &&
            framepointer > stackbottom &&
            i++ < MAX_STACK_DEPTH )
    {   /* now walk up the stack */
        framepointer = (unsigned far *) ( (((long) (sregs.ss)) << 16)
                     + (*framepointer) );
    }

    /* we should be exactly at the top of the stack */
    if (framepointer != stacktop)
    {
        printf ("stack found corrupted in routine %s\n", calling_id);
        return (1);  /* stack bad */
    }
    return (0);  /* stack OK */

}   /* end of check_valid_stack */

/* our dummy main */
void main(argc, argv)
int argc;
char **argv;
{
    unsigned int mainstack = 0; /* used for address */
    stacktop = &mainstack + 1;  /* record address */
    real_main(argc, argv);      /* and call the real main() */
}

#if defined(DRIVER)
/***** the test driver *****/

#define CHECK(x) if(check_valid_stack(x)) exit(1); else printf("%s ok\n", x)
int check_valid_stack(char *);

void subrd()
{
	unsigned far *crash;

	crash = (unsigned far *) &crash;

	/* wipe out stack */
	++crash;	/* skip self */
	*(++crash) = 0;
	*(++crash) = 0;
	*(++crash) = 0;
	*(++crash) = 0;
	*(++crash) = 0;
	*(++crash) = 0;
	CHECK("subrd"); /* should generate an error */
}

void subrc()
{
	CHECK("subrc");
}

void subrb()
{
	int abc;
	char *ptr;

	ptr = 0;
	ptr += abc;
	CHECK("subrb");
	subrc();
	subrd();
}


void subra()
{
	int abc;
	char *ptr;

	ptr = 0;
	ptr += abc;
	CHECK("subra");
	subrb();
}

real_main()
{
	CHECK("main");
	subra();
}
#endif
