***************************************************************
* File:		entry68k.s
*
* Description:	Startup code which starts up the C code.
*
*		This file contains simple start up code for the
*		Intel 68000 family processors in real mode.
*
*		This is a very simple code to set up the vector
*		table and stack and then transfer control to
*		main().  No I/O or heap initialization is done.
*
***************************************************************

entry	IDNT
	section	0
*
* Initialize the most common interrupt vectors to the idle loop
*
	dc.l	stack_top		* 0: reset stack pointer
	dc.l	entry			* 1: reset PC
	dc.l	lstop			* 2: Bus error
	dc.l	lstop			* 3: Address error
	dc.l	lstop			* 4: Illegal instruction
	dc.l	lstop			* 5: Zero divide
	dc.l	lstop			* 6: CHK instruction
	dc.l	lstop			* 7: TRAPV instruction
	dc.l	lstop			* 8: Priviledge violation
	dc.l	lstop			* 9: Trace
	dc.l	lstop			* 10: Line 1010 emulator
	dc.l	lstop			* 11: Line 1111 emulator
	dc.l	lstop			* 12: reserved
	dc.l	lstop			* 13: reserved
	dc.l	lstop			* 14: reserved
	dc.l	lstop			* 15: unitialized interrupt
	dc.l	lstop,lstop,lstop,lstop	* 16-23: reserved
	dc.l	lstop,lstop,lstop,lstop
	dc.l	lstop			* 24: spurious interrupt
	dc.l	lstop			* 25: level 1 autovector
	dc.l	lstop			* 26: level 2 autovector
	dc.l	lstop			* 27: level 3 autovector
	dc.l	lstop			* 28: level 4 autovector
	dc.l	lstop			* 29: level 5 autovector
	dc.l	lstop			* 30: level 6 autovector
	dc.l	lstop			* 31: level 7 autovector
	dc.l	lstop,lstop,lstop,lstop	* 32-47: trap 0-15
	dc.l	lstop,lstop,lstop,lstop
	dc.l	lstop,lstop,lstop,lstop
	dc.l	lstop,lstop,lstop,lstop
	ds.l	208			* reserve space for remaining vectors

	SECTION	9			* Code section

	xref	.main

entry:	MOVE.L	#stack_top,SP		* Iniitialize the stack pointer
	JSR	.main			* Jump to execute main()
*
*	Come here if main() has exited for some reason, or we have found
*	an interrupt that is undefined.
*
lstop:
self:	BRA.S	self
	NOP				* Added to avoid 68020 queue prefetch
	NOP				* problems in this example.
	NOP
	NOP

	section	12
	ds.l	256			* Reserve 1K of stack space
stack_top:

	END       entry
