;**************************************************************
; File:		entry86.asm
;
; Description:	Startup code which starts up the C code.
;
;		This file contains simple start up code for the
;		Intel 80x86 family processors in real mode.
;
;		This is a very simple code to set up the segment
;		registers and stack and then transfer control to
;		main().  No I/O or heap initialization is done.
;
;**************************************************************
;
name	entry86
;
;	Define code and data groups
;
code	group	S__main
data	group	stack
public	__main				; pre-main startup routine
extrn	_main:far			; external reference to main()
;
; Function:	start
;
; Description:	This function is the first function executed after a power
;		up restart.  It is jumped to from the power up execution
;		start up address 0ffff:0.
;
;		This function initializes the code, data, extra, and stack
;		segment registers and the stack pointer.  A "small" model
;		configuration is assumed.  Data, stack, and extra segment
;		registers are all initialized to the same value.
;
S__main	segment	byte public 'code'	; starting code segment
	assume	cs:code,ds:data,ss:data
__main	proc	far
	mov	ax,data			; init data, stack and extra seg regs
	mov	ds,ax
	mov	ss,ax
	mov	es,ax
	lea	sp,stacktop		; intialize stack pointer
	call	far ptr _main		; jump to main()
waitlp:	jmp	waitlp			; where to wait when done
	nop				; avoid prefetch problems
	nop
	nop
	nop
__main	endp
S__main	ends
;
;       Reserve stack space
;
stack	segment word public 'stack'
stackbot label word
	dw	256	dup(?)		; define 256 words of stack
stacktop label word
stack	ends
;
;       Power on reset jump for Intel 80x86 processors.  This must be
;	absolutely loaded at 0ffff:0.
;
poweron	segment at 0ffffh
	jmp far ptr __main		; jump to beginning of program
poweron	ends
  	end __main
