/****************************** VECTS.C ****************************
*   Reports on interrupt vector manipulations
*   by Andrew Binstock, June 1988
*   Small Case. Version 1.0
*   (c) 1988 Andrew Binstock
********************************************************************/
/* Validated for Turbo C, MSC 5.1. For Microsoft, #define MSC 

   Usage:
        vects {-c|-r| -/+99}
        where -c    create virgin interrupt vector table
              -r    report which vectors have been changed
              -/+99 vectors to be changed
*/

#include <stdio.h>
#include <ctype.h>
#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include "vectname.h"

FILE *vect_file;

void show_usage(), set_up_filename(),
     reset_vector(), set_vector(),
     load_vector_table();
int  get_disk();

void far * vects_in_RAM;

struct one_address
{
    unsigned int offset,
                 segment;
}
    vectors[512];

#define YES       1
#define ORIG      0                 /* Tells whether to restore the  */
#define MOD       1                 /* ORIGinal or MODified vector   */
#define MAX_MODS 10                 /* Maximum number of vectors to  */
                                    /* change from one command-line. */

struct v {
            char direction;         /* Either MOD or ORIG            */
            int  number;            /* Vector number                 */
         }
         vects_to_modify [MAX_MODS];/* Array of vectors to modify    */

char * filename = " :\\vectors.dat";

main (int argc, char *argv[])
{
    int j;
    int Create, Modify, Report;
    Create = Modify = Report = 0;

    for (j = 0; j < MAX_MODS; j++)       /* Initialize to any value > 256   */
        vects_to_modify[j].number = 500; /* so as to see which vects to mod */

    if (argc < 2)                        /* User must specify desired action*/
    {
        puts ("No action selected");
        show_usage();
        exit(10);
    }


    while (--argc)
    {
        char *p;
        p = argv[argc];
        if (*p != '-' && *p != '+')
        {
            puts ("Parameters must begin with a '-' or '+'");
            show_usage();
            exit(5);
        }
        else
        {
            static int i;

            char *ch = (isdigit (*(p+1)) ? p : ++p); /* Only advance if '-'
                                                        a letter switch  */
            switch (tolower (*ch))
            {
                case 'c':
                    Create = YES; break;
                case 'r':
                    Report = YES; break;
                case '-':
                case '+':
                {
                    Modify = YES;
                    if (i >= MAX_MODS)
                    {
                        printf ("No more than %d vectors to modify accepted\n",
                                MAX_MODS);
                        exit (9);
                    }

                    if (*p == '-')
                        vects_to_modify[i].direction = ORIG;
                    else
                    if (*p == '+')
                        vects_to_modify[i].direction = MOD;

                    vects_to_modify [i].number = atoi (++p);
                    i += 1;
                    break;
                }
                default:
                {
                    puts ("Error in specifying options.");
                    show_usage();
                    exit (6);
                }
            }
        }
    }

    set_up_filename();

    if (Create)              /* Creates two tables of 256 4-byte vectors */
    {                        /* and writes them out to root directory.   */
        load_vector_table();
        if ((vect_file = fopen (filename, "wb")) == NULL)
        {
            printf ("File: %s could not be created. Program will exit.\n",
                    filename);
            exit (7);
        }
        else
        {
            fwrite (vectors, sizeof (struct one_address), 512, vect_file);
            fclose (vect_file);
        }
    }

    if (Report)             /* Reports any vectors which have been changed*/
    {
        struct one_address * temp, * from_file,  far * ftemp;
        int i, j;

        temp      = malloc (256 * sizeof (struct one_address));
        from_file = malloc (256 * sizeof (struct one_address));

        if (temp == NULL || from_file == NULL)
        {
            puts ("Cannot allocate memory for vector analysis");
            exit (8);
        }

        ftemp = (void far *) temp;

        movedata (FP_SEG (vects_in_RAM), FP_OFF (vects_in_RAM),
                  FP_SEG (ftemp),        FP_OFF (ftemp),        1024);

        if ((vect_file = fopen (filename, "rb")) == NULL)
        {
            printf ("File: %s could not be read. Program will exit.\n",
                    filename);
            exit (7);
        }

        fread (from_file, sizeof (struct one_address), 256, vect_file);
        fclose (vect_file);

        for (i = j = 0; i < 256; i++)
        {
            if (temp[i].offset  != from_file[i].offset ||
                temp[i].segment != from_file[i].segment)
            {
                printf ("%3i was %04X:%04X is %04X:%04X %s\n", i,
                    from_file[i].segment, from_file[i].offset,
                    temp[i].segment, temp[i].offset,
                    (i < 104 ? vect_name[i] : " "));
                j++;
            }
        }

        if (j == 0)
            puts ("No changed vectors found");
    }   /* End of Report */

    if (Modify)                   /* Copy in vectors from the file,   */
    {                             /* then modify vectors as requested */
        int i;

        if ((vect_file = fopen (filename, "r+b")) == NULL)
        {
            printf ("File: %s could not be read. Program will exit.\n",
                    filename);
            exit (7);
        }

        fread (vectors, sizeof (struct one_address), 512, vect_file);

        for (i = 0; i < MAX_MODS; i++) /* Step thru array of vects to modify */
        {
            if (vects_to_modify[i].number == 500)  /* Test for initial value */
                break;
            fseek (vect_file, 0L, SEEK_SET);       /* Reset to top of file   */
            reset_vector (vects_to_modify[i].number,
                          vects_to_modify[i].direction);
        }
        fclose (vect_file);
    }

    return (0);
}

void set_up_filename()
{
    int this_disk = get_disk();
    *filename = ((char)(this_disk + 'A'));
}

int get_disk()  /* Returns current disk in form 0 = A, 1 = B, etc. */
{
    int curr_disk;
#if defined __TURBOC__
    curr_disk = getdisk();
#elif defined MSC
    _dos_getdrive((unsigned int *) &curr_disk);
    curr_disk -= 1;         /* Microsoft use A = 1, B = 2, etc. */
#endif

    return (curr_disk > 2 ? 2 : curr_disk); /* If it's a hard disk,     */
}                                           /* make sure we only use C: */

void reset_vector (int vector, int target)
{
    if (target == ORIG)                     /* Restore original vector   */
    {                                       /* saving the modified value */
        struct one_address far * p = vectors + 256 + vector;
        movedata (0, (unsigned) vector * 4, FP_SEG (p), FP_OFF (p), 4);
        set_vector (vector, vectors[vector].segment, vectors[vector].offset);
                                            /* Write the changed vector tbl */
        fwrite (vectors, sizeof (struct one_address), 512, vect_file);
    }
    else
    if (target == MOD)                      /* Restore modified vector    */
    {                                       /* but no unmodiifed ones.    */
        if (vectors[vector].segment == vectors[vector + 256].segment &&
            vectors[vector].offset  == vectors[vector + 256].offset)
            printf ("Vector %d was not previously changed\n", vector);
        else
            set_vector (vector, vectors[vector + 256].segment,
                                vectors[vector + 256].offset);
    }
}

void set_vector (int which_vector, unsigned segment, unsigned offset)
{
    union  REGS  r;
    struct SREGS s;

    r.h.ah = 0x25;
    r.h.al = which_vector;
    r.x.dx = offset;
    s.ds   = segment;
    int86x (0x21, &r, &r, &s);
}

void load_vector_table()     /* Creates two tables of 256 4-byte vectors */
{                            /* First is original, 2nd for modifications */

    void far * table_ptr = (void far *) &vectors[0];
    vects_in_RAM = (void far *) NULL;

    movedata (FP_SEG (vects_in_RAM), FP_OFF (vects_in_RAM),
                FP_SEG (table_ptr),    FP_OFF (table_ptr),   1024);

    table_ptr = (void far *) &vectors[256];

    movedata (FP_SEG (vects_in_RAM), FP_OFF (vects_in_RAM),
                FP_SEG (table_ptr),    FP_OFF (table_ptr),   1024);
}

void show_usage()
{
    puts ("vects -c -r +/-99");
    puts ("\twhere -c creates a table of current vectors for later use");
    puts ("\t      -r reports the redirected vectors");
    puts ("\t      where 99 is any vector number (decimal) 0 - 255");
    puts ("\t         -99 restores original int 99 address");
    puts ("\t         +99 restores modified int 99 address");
}
